# Copyright 2004 James Bunton <james@delx.cjb.net>
# Licensed for distribution under the GPL version 2, check COPYING for details

from twisted.protocols import msn
import groupchat
import msnw
import config
import debug


def msnlist2jabsub(lists):
	if(lists & (msn.FORWARD_LIST | msn.REVERSE_LIST)):
		return "both"
	elif(lists & (msn.REVERSE_LIST)):
		return "from"
	elif(lists & (msn.FORWARD_LIST)):
		return "to"
	else:
		return None

def jabsub2msnlist(sub):
	if(sub == "to"):
		return msn.FORWARD_LIST
	elif(sub == "from"):
		return msn.REVERSE_LIST
	elif(sub == "both"):
		return (msn.FORWARD_LIST | msn.REVERSE_LIST)
	else:
		return 0

def msn2jid(msnid):
	# FIXME, hardcoded
	return msnid.replace('@', '%') + "@" + config.jid

translateAccount = msn2jid

def jid2msn(jid):
	return unicode(jid[:jid.find('@')].replace('%', '@'))

def show2state(show): 
	""" Converts a Jabber show into an MSN status code """
	if(show in [None, "online", "chat"]):
		return msn.STATUS_ONLINE
	elif(show in ["dnd"]):
		return msn.STATUS_BUSY
	elif(show in ["away", "xa"]):
		return msn.STATUS_AWAY
	else:
		return msn.STATUS_OFFLINE


def state2show(state):
	if(state == msn.STATUS_ONLINE):
		return "online"
	elif(state == msn.STATUS_BUSY):
		return "dnd"
	elif(state == msn.STATUS_AWAY):
		return "away"
	elif(state == msn.STATUS_IDLE):
		return "away"
	elif(state == msn.STATUS_BRB):
		return "away"
	elif(state == msn.STATUS_PHONE):
		return "dnd"
	elif(state == msn.STATUS_LUNCH):
		return "away"
	else:
		return None


#Possible entry points for groupchat
#	- User starts an empty switchboard session by sending presence to a blank room
#	- An existing switchboard session is joined by another MSN user
#	- User invited to an existing switchboard session with more than one user


class LegacyGroupchat(groupchat.BaseGroupchat):
	def __init__(self, session, resource, ID=None, existing=False, switchboardSession=None):
		groupchat.BaseGroupchat.__init__(self, session, resource, ID)
		if(not existing):
			self.switchboardSession = msnw.GroupchatSwitchboardSession(self, makeSwitchboard=True)
		else:
			self.switchboardSession = switchboardSession
			
		assert(self.switchboardSession != None)
		
		debug.log("LegacyGroupchat: \"%s\" created" % (self.roomJID()))
	
	def removeMe(self):
		self.switchboardSession.removeMe()
		self.switchboardSession = None
		groupchat.BaseGroupchat.removeMe(self)
		debug.log("LegacyGroupchat: \"%s\" destroyed" % (self.roomJID()))
	
	def sendLegacyMessage(self, message):
		debug.log("LegacyGroupchat: \"%s\" sendLegacyMessage(\"%s\")" % (self.roomJID(), message))
		self.switchboardSession.sendMessage(message)
	
	def sendContactInvite(self, contactJID):
		debug.log("LegacyGroupchat: \"%s\" sendContactInvite(\"%s\")" % (self.roomJID(), contactJID))
		userHandle = jid2msn(contactJID)
		self.switchboardSession.inviteUser(userHandle)



class LegacyConnection(msnw.MSNConnection):
	""" A glue class that connects to the legacy network """
	def __init__(self, username, password, session):
		msnw.MSNConnection.__init__(self, username, password)
		self.session = session
		self.listSynced = False
		import subscription # Is in here to prevent an ImportError loop
		self.subscriptions = subscription.SubscriptionManager(self.session)
		debug.log("LegacyConnection: \"%s\" - created" % (self.session.jabberID))
	
	def removeMe(self):
		debug.log("LegacyConnection: \"%s\" - being deleted" % (self.session.jabberID))
		
		if(self.getContacts()):
			for userHandle in self.getContacts().getContacts():
				msnContact = self.getContacts().getContact(userHandle)
				if(msnContact.status != msn.STATUS_OFFLINE):
					msnContact.status = msn.STATUS_OFFLINE
					self.sendMSNContactPresence(msnContact)
		
		msnw.MSNConnection.changeStatus(self, msn.STATUS_OFFLINE, self.session.nickname) # Change our nickname on the MSN network (stops users from appearing offline with a nickname "james - Online")
		
		msnw.MSNConnection.removeMe(self)
		self.subscriptions.removeMe()
		self.subscriptions = None
		self.session = None
	
	def sendMessage(self, dest, body):
		dest = jid2msn(dest)
		msnw.MSNConnection.sendMessage(self, dest, body)
	
 	def setStatus(self, show, friendly):
		statusCode = show2state(show)
		msnw.MSNConnection.changeStatus(self, statusCode, friendly)
	
	def newResourceOnline(self, resource):
		self.sendLists(resource)
	
	def jabberSubscriptionReceived(self, source, subtype):
		self.subscriptions.jabberSubscriptionReceived(source, subtype)
	
	def sendMSNContactPresence(self, msnContact, to=None):
		if(not to):
			to = self.session.jabberID
		source = msn2jid(msnContact.userHandle)
		show = state2show(msnContact.status)
		ptype = None
		if(not show):
			ptype = "unavailable"
		status = msnContact.screenName.decode("utf-8")
		self.session.sendPresence(to=to, fro=source, show=show, status=status, ptype=ptype)
	
	def sendMSNUserPresence(self, userHandle, to=None):
		msnContact = self.getContacts().getContact(userHandle)
		self.sendMSNContactPresence(msnContact, to)
	
	def sendLists(self, resource):
		""" Sends a copy of the MSN contact presences to this resource """
		debug.log("LegacyConnection: \"%s\" - sendLists(\"%s\")" % (self.session.jabberID, resource))
		fulljid = self.session.jabberID
		if(resource):
			fulljid += "/" + resource
		if(self.getContacts()):
			for userHandle in self.getContacts().getContacts():
				self.sendMSNUserPresence(userHandle, fulljid)
	
	def listSynchronized(self):
		self.session.sendPresence(to=self.session.jabberID, fro=config.jid)
		self.subscriptions.syncJabberLegacyLists()
		self.listSynced = True
		self.subscriptions.flushSubscriptionBuffer()
	
	def gotMessage(self, remoteUser, text):
		source = msn2jid(remoteUser)
		text = text.decode("utf-8")
		self.session.sendMessage(to=self.session.jabberID, fro=source, body=text, mtype="chat")
	
	def loggedIn(self):
		if(self.session):
			debug.log("LegacyConnection: \"%s\" - loggedIn()" % (self.session.jabberID))
			self.session.ready = True
	
	def contactStatusChanged(self, remoteUser):
		if(self.session): # Make sure the transport isn't shutting down
			debug.log("LegacyConnection: \"%s\" - contactStatusChanged(\"%s\")" % (self.session.jabberID, remoteUser))
			self.sendMSNUserPresence(remoteUser)
	
	def ourStatusChanged(self, statusCode):
		# Send out a new presence packet to the Jabber user so that the MSN-t icon changes
		source = config.jid
		to = self.session.jabberID
		show = state2show(statusCode)
		debug.log("LegacyConnection: \"%s\" - ourStatusChanged(\"%s\")" % (self.session.jabberID, statusCode))
		self.session.sendPresence(to=to, fro=source, show=show)
	
	def userAddedMe(self, userHandle):
		self.subscriptions.msnContactAddedMe(userHandle)
	
	def userRemovedMe(self, userHandle):
		self.subscriptions.msnContactRemovedMe(userHandle)
	
	def serverGoingDown(self):
		if(self.session):
			self.session.sendMessage(to=self.session.jabberID, fro=config.jid, body=config.msnMaintenance)
	
	def multipleLogin(self):
		if(self.session):
			self.session.sendMessage(to=self.session.jabberID, fro=config.jid, body=msnMultipleLogin)
			self.session.removeMe()
	
	def accountNotVerified(self):
		if(self.session):
			text = config.msnNotVerified % (self.session.username)
			self.session.sendMessage(to=self.session.jabberID, fro=config.jid, body=text)
	
	def loginFailure(self, message):
		if(self.session):
			text = config.msnLoginFailure % (self.session.username)
			self.session.sendMessage(to=self.session.jabberID, fro=config.jid, body=text)
	
	def failedMessage(self, remoteUser, message):
		if(self.session):
			fro = msn2jid(remoteUser)
			text = config.msnFailedMessage + message
			self.session.sendMessage(to=self.session.jabberID, fro=fro, body=text)
	
	def initialEmailNotification(self, inboxunread, foldersunread):
		text = config.msnInitialMail % (inboxunread, foldersunread)
		self.session.sendMessage(to=self.session.jabberID, fro=config.jid, body=text)
	
	def realtimeEmailNotification(self, mailfrom, fromaddr, subject):
		text = config.msnRealtimeMail % (mailfrom, fromaddr, subject)
		self.session.sendMessage(to=self.session.jabberID, fro=config.jid, body=text)
		
	def connectionLost(self, reason):
		if(self.session):
			debug.log("LegacyConnection: \"%s\" - connectionLost(\"%s\")" % (self.session.jabberID, reason))
			text = config.msnDisconnected % (reason)
			self.session.sendMessage(to=self.session.jabberID, fro=config.jid, body=text)
			self.session.removeMe() # Tear down the session
